import numpy as np
from ..main import SapphireRenderer


class ParticleManager:
    def __init__(self, parent_object, renderer: SapphireRenderer, hide_parent=True):
        self.particles = []
        self.parent_object = parent_object
        self.renderer = renderer

        if hide_parent:
            parent_object.hide()

    def add_particle(self, position=np.array([0.0, 0.0, 0.0]), color=(0, 0, 0)):
        """
        Add a particle to the particle manager
        :param position: the position of the particle
        :param color: the color of the particle
        :return: the particle object
        """
        if not hasattr(self.parent_object, "copy"):
            raise AttributeError(
                "The parent object must have a copy method to be used as a particle"
            )

        particle = self.renderer.direct_add_object(self.parent_object.copy())
        particle.move_absolute(position)
        particle.set_color(color)

        self.particles.append(particle)

        return particle

    def remove_particle(self, particle):
        """
        Remove a particle from the particle manager
        :param particle: the particle to remove
        """
        self.particles.remove(particle)
        self.renderer.remove_object(particle)

    def clear_particles(self):
        """
        Remove all particles from the particle manager
        """
        for particle in self.particles:
            self.renderer.remove_object(particle)
        self.particles = []

    def move_particles(self, movement):
        """
        Move all particles by a certain amount
        :param movement: the amount to move the particles by
        """
        for particle in self.particles:
            particle.move_relative(movement)



import numpy as np
from numba import njit


@njit(fastmath=True)
def project_point(point, offset_array, focal_length):
    if point[1] < 0:
        return None, 1

    focal_length_divided_by_y = focal_length / point[1]

    projected_point = np.array(
        [
            point[0] * focal_length_divided_by_y,
            -point[2] * focal_length_divided_by_y,
        ]
    )

    projected_point += offset_array

    scale_factor = 1 / np.linalg.norm(point)

    return projected_point, scale_factor
